// Copyright (c) 1999-2015  Pavel Rychly

#ifndef FINLIB_GENERATOR_HH
#define FINLIB_GENERATOR_HH


template <class Value>
class Generator {
protected:
    Generator() {}
public:
    Value operator()() {return next();}
    virtual Value next() = 0;
    virtual bool end() = 0;
    virtual int size() {return 1;}
    virtual ~Generator() {}
};


template <class Value>
class EmptyGenerator: public Generator<Value> {
protected:
    const Value val; 
public:
    EmptyGenerator (Value v=Value()): val(v) {}
    virtual Value next() {return val;}
    virtual bool end() {return true;}
    virtual int size() {return 0;}
    virtual ~EmptyGenerator() {}
};


template <class Value>
class SequenceGenerator: public Generator<Value> {
protected:
    Value curr; 
    const Value last; 
public:
    SequenceGenerator (Value first, Value last): curr(first), last(last) {}
    virtual Value next() {return curr++;}
    virtual bool end() {return curr > last;}
    virtual int size() {return last - curr +1;}
    virtual ~SequenceGenerator() {}
};

template <class Value>
class ArrayGenerator: public Generator<Value> {
protected:
    Value *array, *curr, *last;
public:
    ArrayGenerator (Value *a, unsigned size):
        array (a), curr (a), last (a + size) {}
    virtual Value next() {return *curr++;}
    virtual bool end() {return curr >= last;}
    virtual int size() {return last - curr;}
    virtual ~ArrayGenerator() {delete[] array;}
};



#endif

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
